!= Ĥʪ̤򤢤ͤĴ
!
!= Adjust some physical values to a certain fixed values
!
! Authors::   Yasuhiro MORIKAWA
! Version::   $Id: phy_adjust_fixed.f90,v 1.2 2008-06-01 16:06:53 morikawa Exp $
! Tag Name::  $Name: dcpam4-20080609-1 $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]   
!

module phy_adjust_fixed
  !
  != Ĥʪ̤򤢤ͤĴ
  !
  != Adjust some physical values to a certain fixed values
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! Ĥʪ (ߤϲ٤漾Τ) 򤢤ͤ˸ꤷޤ. 
  ! κ, ͤȽͤκ⻻Фޤ. 
  !
  ! Fix some physical values (now, only temperature and specific humidity)
  ! to a certain value. 
  ! And calculate difference between original values and fixed values. 
  !
  !== Procedures List
  !
  ! PhyAdjFixCreate        :: PHYADJFIX ѿν
  ! AdjustFixed            :: 黻
  ! PhyAdjFixClose         :: PHYADJFIX ѿνλ
  ! PhyAdjFixPutLine       :: PHYADJFIX ѿ˳ǼƤΰ
  ! PhyAdjFixinitialized   :: PHYADJFIX ѿꤵƤ뤫ݤ
  ! PhyAdjFixSetTime       :: 
  ! ------------  :: ------------
  ! PhyAdjFixCreate        :: Constructor of "PHYADJFIX"
  ! AdjustFixed            :: Calculation
  ! PhyAdjFixClose         :: Deconstructor of "PHYADJFIX"
  ! PhyAdjFixPutLine       :: Print information of "PHYADJFIX"
  ! PhyAdjFixinitialized   :: Check initialization of "PHYADJFIX"
  ! PhyAdjFixSetTime       :: Configure time
  !
  !== Usage
  !
  ! Ϥ, PHYADJFIX ѿ, 
  ! PhyAdjFixCreate ǽԤޤ.
  ! AdjustFixed ʪ (ߤϲ٤漾Τ) Ϳ뤳Ȥ, 
  ! 줾򤢤 (PhyAdjFixCreate ˤƻ) ظꤷޤ. 
  ! PhyAdjFixCreate νǽԤ, 
  ! ͤȽͤȤκϤޤ. 
  ! PHYADJFIX ѿνλˤ
  ! PhyAdjFixClose ѤƤ.
  !
  ! First, initialize "PHYADJFIX" by "PhyAdjFixCreate".
  ! Fix physical values (now, only temperature and specific humidity)
  ! by given them to "AdjustFixed". 
  ! When output settings are done within initialization of 
  ! "PhyAdjFixCreate", difference between original values 
  ! and fixed values is output. 
  ! In order to terminate "PHYADJFIX", use "PhyAdjFixClose".
  !

  use dc_types, only: DP, TOKEN, STRING
  use dc_date_types, only: DC_DIFFTIME
  use gt4_history_nmlinfo, only: GTHST_NMLINFO
  implicit none
  private
  public:: PHYADJFIX
  public:: PhyAdjFixCreate, AdjustFixed
  public:: PhyAdjFixClose, PhyAdjFixPutLine
  public:: PhyAdjFixInitialized, PhyAdjFixSetTime

  type PHYADJFIX
    !
    ! ޤ, PhyAdjFixCreate  "PHYADJFIX" 
    ! ѿꤷƲ. 
    ! ꤵ줿 "PHYADJFIX" ѿѤݤˤ,
    ! PhyAdjFixClose ˤäƽλԤäƤ.
    !
    ! Initialize "PHYADJFIX" variable by 
    ! "PhyAdjFixCreate" before usage.
    ! If you reuse "PHYADJFIX" variable again for another application, 
    ! terminate by "PhyAdjFixClose".
    !
    logical:: initialized = .false.
                              ! ե饰. 
                              ! Initialization flag

    !-----------------------------------------------------------------
    !  ʻȿ
    !  Grid points and maximum truncated wavenumber
    !-----------------------------------------------------------------
    integer:: imax            ! ٳʻ. 
                              ! Number of grid points in longitude
    integer:: jmax            ! ٳʻ. 
                              ! Number of grid points in latitude
    integer:: kmax            ! ľؿ. 
                              ! Number of vertical level

    !-----------------------------------------------------------------
    !  ǡ
    !  Axes data
    !-----------------------------------------------------------------
    real(DP), pointer:: x_Lon (:) =>null()
                              ! . Longitude
    real(DP), pointer:: x_Lon_Weight (:) =>null()
                              ! ʬѺɸŤ. 
                              ! Weight for integration in longitude
    real(DP), pointer:: y_Lat (:) =>null()
                              ! . Latitude
    real(DP), pointer:: y_Lat_Weight (:) =>null()
                              ! ʬѺɸŤ. 
                              ! Weight for integration in latitude
    real(DP), pointer:: z_Sigma (:) =>null()
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), pointer:: z_DelSigma (:) =>null()
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)

    !-----------------------------------------------------------------
    !  
    !  Coefficients
    !-----------------------------------------------------------------
!!$    real(DP):: CoefAlpha      ! $ \alpha $ . . Coefficient

    !-----------------------------------------------------------------
    !  
    !  Fixed values
    !-----------------------------------------------------------------
    real(DP):: TempFixed = -1.0_DP
                              ! $ T_0 $ .     ٤θ. 
                              ! Fixed value of temperature
    real(DP):: QVapFixed = -1.0_DP
                              ! $ q_0 $ .     漾θ. 
                              ! Fixed value of specific humidity
    real(DP), pointer:: xyz_TempFixed (:,:,:) =>null()
                              ! $ T_0 $ .     ٤θ. 
                              ! Fixed value of temperature
    real(DP), pointer:: xyz_QVapFixed (:,:,:) =>null()
                              ! $ q_0 $ .     漾θ. 
                              ! Fixed value of specific humidity

    !-----------------------------------------------------------------
    !  ʸǡ
    !  Character data
    !-----------------------------------------------------------------
!!$    character(TOKEN):: key00  ! . Keyword
!!$

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    type(DC_DIFFTIME):: current_time
                              ! ߻. Current time.
    type(DC_DIFFTIME):: delta_time
                              ! $ \Delta t $ . ॹƥå. Time step

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------
    type(GTHST_NMLINFO):: gthstnml
                              ! NAMELIST#phy_adjust_fixed_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#phy_adjust_fixed_history_nml". 
  end type PHYADJFIX


  character(*), parameter:: version = &
    & '$Name: dcpam4-20080609-1 $' // &
    & '$Id: phy_adjust_fixed.f90,v 1.2 2008-06-01 16:06:53 morikawa Exp $'

  !-----------------------------------------------------------------
  !  ³
  !  Public procedures
  !-----------------------------------------------------------------

  interface PhyAdjFixCreate
    module procedure PhyAdjFixCreate
  end interface

  interface AdjustFixed
    module procedure AdjustFixed
  end interface

!!$  interface PhyAdjFixSample
!!$    module procedure PhyAdjFixSample
!!$  end interface

  interface PhyAdjFixClose
    module procedure PhyAdjFixClose
  end interface

  interface PhyAdjFixPutLine
    module procedure PhyAdjFixPutLine
  end interface

  interface PhyAdjFixinitialized
    module procedure PhyAdjFixInitialized
  end interface

  interface PhyAdjFixSetTime
    module procedure PhyAdjFixSetTime
  end interface

  !-----------------------------------------------------------------
  !  ³
  !  Private procedures
  !-----------------------------------------------------------------

  interface NmlRead
    module procedure PhyAdjFixNmlRead
  end interface

contains

  subroutine PhyAdjFixCreate( phy_adj_fix, &
    & imax, jmax, kmax, &
    & x_Lon, y_Lat, z_Sigma, &
    & DelTime, &
    & x_Lon_Weight, y_Lat_Weight, z_DelSigma, &
    & TempFixed, QVapFixed, &
    & xyz_TempFixed, xyz_QVapFixed, &
    & current_time_value, current_time_unit, &
    & history_varlist, &
    & history_interval_value, history_interval_unit, &
    & history_precision, history_fileprefix, &
    & nmlfile, err )
    !
    ! PHYADJFIX ѿνԤޤ. 
    ! ¾Υ֥롼ѤɬΥ֥롼ˤä 
    ! PHYADJFIX ѿꤷƤ. 
    !
    ! ʤ, Ϳ줿 *phy_adj_fix* ˽ꤵƤ, 
    ! ץϥ顼ȯޤ. 
    !
    ! NAMELIST Ѥˤϰ *nmlfile*  NAMELIST ե̾
    ! ͿƤ. NAMELIST ѿξܺ٤˴ؤƤ 
    ! NAMELIST#phy_adjust_fixed_nml 򻲾ȤƤ. 
    !
    ! Constructor of "PHYADJFIX". 
    ! Initialize *phy_adj_fix* by this subroutine, 
    ! before other procedures are used, 
    !
    ! Note that if *phy_adj_fix* is already initialized 
    ! by this procedure, error is occurred. 
    !
    ! In order to use NAMELIST, specify a NAMELIST filename to 
    ! argument *nmlfile*. See "NAMELIST#phy_adjust_fixed_nml" 
    ! for details about a NAMELIST group. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_present, only: present_and_not_empty, present_and_true, &
      & present_select
    use dc_message, only: MessageNotify
    use dcpam_error, only: StoreError, DCPAM_EARGSDUPLICATE
    use dc_error, only: DC_NOERR, DC_EALREADYINIT, &
      & DC_EARGLACK, DC_ENEGATIVE, DC_ENOFILEREAD, USR_ERRNO, HST_EBADVARNAME
    use dc_date, only: DCDiffTimeCreate
    use dc_hash, only: HASH, DCHashPut, DCHashRewind, DCHashNext, DCHashDelete
    use gt4_history_nmlinfo, only: HstNmlInfoCreate, HstNmlInfoAdd, &
      & HstNmlInfoEndDefine, HstNmlInfoPutLine, HstNmlInfoAllNameValid
    use gt4_history, only: GT_HISTORY, &
      & HistoryAddVariable, HistoryAddAttr
    implicit none
    type(PHYADJFIX), intent(inout):: phy_adj_fix
    integer, intent(in):: imax ! ٳʻ. 
                               ! Number of grid points in longitude
    integer, intent(in):: jmax ! ٳʻ. 
                               ! Number of grid points in latitude
    integer, intent(in):: kmax ! ľؿ. 
                               ! Number of vertical level
    real(DP), intent(in):: x_Lon (0:imax-1)
                              ! . Longitude
    real(DP), intent(in):: y_Lat (0:jmax-1)
                              ! . Latitude
    real(DP), intent(in):: z_Sigma (0:kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), intent(in):: DelTime    ! $ \Delta t $ . ॹƥå. Time step
    real(DP), intent(in), optional:: x_Lon_Weight (0:imax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in longitude
    real(DP), intent(in), optional:: y_Lat_Weight (0:jmax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in latitude
    real(DP), intent(in), optional:: z_DelSigma (0:kmax-1)
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)
    real(DP), intent(in), optional:: TempFixed
                              ! $ T_0 $ .     ٤θ (Ѳ̵). 
                              ! Fixed value of temperature (spatial variation is nothing)
    real(DP), intent(in), optional:: QVapFixed
                              ! $ q_0 $ .     漾θ (Ѳ̵). 
                              ! Fixed value of specific humidity (spatial variation is nothing)
    real(DP), intent(in), optional:: xyz_TempFixed (0:imax-1,0:jmax-1,0:kmax-1)
                              ! $ T_0 $ .     ٤θ. 
                              ! Fixed value of temperature
    real(DP), intent(in), optional:: xyz_QVapFixed (0:imax-1,0:jmax-1,0:kmax-1)
                              ! $ q_0 $ .     漾θ. 
                              ! Fixed value of specific humidity

    real, intent(in), optional:: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in), optional:: current_time_unit
                              ! ߻ñ. Unit of current time
    character(*), intent(in), optional:: history_varlist
                              ! ҥȥǡνѿꥹ. 
                              ! ޤǶڤä¤٤. 
                              ! (: "Data1,Data2" ). 
                              ! 
                              ! List of variables output to history data. 
                              ! Delimiter is comma. 
                              ! (exp. "Data1,Data2" ). 
                              ! 
    real, intent(in), optional:: history_interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! Numerical value for interval of history data output
    character(*), intent(in), optional:: history_interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(*), intent(in), optional:: history_precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    character(*), intent(in), optional:: history_fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefix of history data filenames

    character(*), intent(in), optional:: nmlfile
                              ! NAMELIST ե̾. 
                              ! ΰ˶ʸʳͿ, 
                              ! ꤵ줿ե뤫 
                              ! NAMELIST ѿɤ߹ߤޤ. 
                              ! եɤ߹ʤˤϥ顼
                              ! ޤ. 
                              !
                              ! NAMELIST ѿξܺ٤˴ؤƤ 
                              ! NAMELIST#phy_adjust_fixed_nml 
                              ! 򻲾ȤƤ. 
                              !
                              ! NAMELIST file name. 
                              ! If nonnull character is specified to 
                              ! this argument, 
                              ! NAMELIST group name is loaded from the 
                              ! file. 
                              ! If the file can not be read, 
                              ! an error occurs.
                              ! 
                              ! See "NAMELIST#phy_adjust_fixed_nml" 
                              ! for details about a NAMELIST group. 
                              ! 

    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: longname = ''
                              ! ѿεŪ̾. Descriptive name of variables
    character(STRING), allocatable:: dims(:)
                              ! ɸ̾. Name of axes
    character(STRING):: units = ''
                              ! ñ. Units
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    type(HASH):: registered_varnames
                              ! Υ⥸塼뤫Ϥѿ̾Υꥹ.
                              ! 
                              ! List of names of variables output 
                              ! from this module. 
    logical:: hash_end
                              ! HASH ֥Ȥνλե饰. 
                              ! 
                              ! End flag of "HASH" object
    logical:: allvar_invalid
                              ! ̵ѿ̾Υåե饰. 
                              ! 
                              ! Check flag of invalid variable names
    character(STRING):: names_invalid
                              ! ̵ѿ̾. 
                              ! 
                              ! Invalid variable names
    character(STRING):: nmlfile_msg
                              ! NAMELIST ե̾˴ؤå. 
                              ! 
                              ! Messages about NAMELIST file name

    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP), parameter:: PI = 3.1415926535897930_DP
                                ! $ \pi $ . ߼Ψ. Circular constant

    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhyAdjFixCreate'
  continue
    call BeginSub( subname, version )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( phy_adj_fix % initialized ) then
      stat = DC_EALREADYINIT
      cause_c = 'PHYADJFIX'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  Υå
    !  Validate arguments
    !-----------------------------------------------------------------
    if (imax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'imax'
      goto 999
    end if
    if (jmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'jmax'
      goto 999
    end if
    if (kmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'kmax'
      goto 999
    end if
    if (DelTime < 0.0_DP) then
      stat = DC_ENEGATIVE
      cause_c = 'DelTime'
      goto 999
    end if

    if ( present( TempFixed ) .and. present( xyz_TempFixed ) ) then
      stat = DCPAM_EARGSDUPLICATE
      cause_c = 'TempFixed,xyz_TempFixed'
      goto 999
    end if

    if ( present( QVapFixed ) .and. present( xyz_QVapFixed ) ) then
      stat = DCPAM_EARGSDUPLICATE
      cause_c = 'QVapFixed,xyz_QVapFixed'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ȿʻ
    !  Configure wave number and grid point
    !-----------------------------------------------------------------
    phy_adj_fix % imax  = imax 
    phy_adj_fix % jmax  = jmax 
    phy_adj_fix % kmax  = kmax 

    !-----------------------------------------------------------------
    !  ɸ
    !  Configure axes
    !-----------------------------------------------------------------
    allocate( phy_adj_fix % x_Lon (0:imax-1) )
    phy_adj_fix % x_Lon = x_Lon

    allocate( phy_adj_fix % y_Lat (0:jmax-1) )
    phy_adj_fix % y_Lat = y_Lat

    allocate( phy_adj_fix % z_Sigma (0:kmax-1) )
    phy_adj_fix % z_Sigma = z_Sigma

    allocate( phy_adj_fix % x_Lon_Weight (0:imax-1) )
    if ( present(x_Lon_Weight) ) then
      phy_adj_fix % x_Lon_Weight = x_Lon_Weight
    else
      phy_adj_fix % x_Lon_Weight = 2.0_DP * PI / imax
    end if

    allocate( phy_adj_fix % y_Lat_Weight (0:jmax-1) )
    if ( present(y_Lat_Weight) ) then
      phy_adj_fix % y_Lat_Weight = y_Lat_Weight
    else
      phy_adj_fix % y_Lat_Weight = 2.0_DP / jmax
    end if

    allocate( phy_adj_fix % z_DelSigma (0:kmax-1) )
    if ( present(z_DelSigma) ) then
      phy_adj_fix % z_DelSigma = z_DelSigma
    else
      phy_adj_fix % z_DelSigma = 1.0_DP / kmax
    end if

    !-----------------------------------------------------------------
    !  ͤ
    !  Fixed values are configured
    !-----------------------------------------------------------------
    if ( present(TempFixed) ) then
      allocate( phy_adj_fix % xyz_TempFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      phy_adj_fix % xyz_TempFixed = TempFixed
    elseif ( present(xyz_TempFixed) ) then
      allocate( phy_adj_fix % xyz_TempFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      phy_adj_fix % xyz_TempFixed = xyz_TempFixed
    end if

    if ( present(QVapFixed) ) then
      allocate( phy_adj_fix % xyz_QVapFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      phy_adj_fix % xyz_QVapFixed = QVapFixed
    elseif ( present(xyz_QVapFixed) ) then
      allocate( phy_adj_fix % xyz_QVapFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      phy_adj_fix % xyz_QVapFixed = xyz_QVapFixed
    end if

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    if ( present(current_time_value) .and. present(current_time_unit) ) then
      call DCDiffTimeCreate( &
        & diff = phy_adj_fix % current_time, &  ! (out)
        & value = real( current_time_value, DP ), & ! (in)
        & unit = current_time_unit )                ! (in)
    else
      call DCDiffTimeCreate( &
        & diff = phy_adj_fix % current_time, & ! (out)
        & value = 0.0_DP, &    ! (in)
        & unit = 'sec' )       ! (in)
    end if

    call DCDiffTimeCreate( &
      & diff = phy_adj_fix % delta_time, & ! (out)
      & value = DelTime, &             ! (in)
      & unit = 'sec' )                 ! (in)

    !-----------------------------------------------------------------
    !  "PHYADJFIX" 
    !  Configure the settings for "PHYADJFIX"
    !-----------------------------------------------------------------
!!$    phy_adj_fix % CoefAlpha = CoefAlpha
!!$    phy_adj_fix % key00     = ''

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------

    !-------------------------
    !  ǥե
    !  Default values
    call HstNmlInfoCreate( gthstnml = phy_adj_fix % gthstnml ) ! (inout)

    !-------------------------
    !  ץʥ
    !  Values from optional arguments
    call HstNmlInfoAdd( &
      & gthstnml = phy_adj_fix % gthstnml, &   ! (inout)
      & name = '', &                               ! (in)
      & interval_value = history_interval_value, & ! (in)
      & interval_unit = history_interval_unit, &   ! (in)
      & precision = history_precision, &           ! (in)
      & average = .false., &                       ! (in)
      & fileprefix = history_fileprefix )          ! (in)

    if ( present(history_varlist) ) then
      call HstNmlInfoAdd( &
        & gthstnml = phy_adj_fix % gthstnml, & ! (inout)
        & name = history_varlist )                 ! (in)
    end if

    !-----------------------------------------------------------------
    !  NAMELIST ͤɤ߹
    !  Load values from NAMELIST
    !-----------------------------------------------------------------
    if ( present_and_not_empty(nmlfile) ) then
      call MessageNotify( 'M', subname, &
        & 'Loading NAMELIST file "%c" ...', &
        & c1 = trim(nmlfile) )
      call NmlRead ( nmlfile = nmlfile, &          ! (in)
        & TempFixed   = phy_adj_fix % TempFixed, & ! (inout)
        & QVapFixed   = phy_adj_fix % QVapFixed, & ! (inout)
!!$        & key00_      = phy_adj_fix % key00, &     ! (inout)
        & gthstnml    = phy_adj_fix % gthstnml, &  ! (inout)
        & err = err )                              ! (out)
      if ( present_and_true(err) ) then
        call MessageNotify( 'W', subname, &
          & '"%c" can not be read.', &
          & c1 = trim(nmlfile) )
        stat = DC_ENOFILEREAD
        cause_c = nmlfile
        goto 999
      end if
    end if

    call HstNmlInfoEndDefine( &
      & gthstnml = phy_adj_fix % gthstnml, & ! (inout)
      & err = err )                              ! (out)
    if ( present_and_true( err ) ) then
      stat = USR_ERRNO
      goto 999
    end if

    if ( .not. ( phy_adj_fix % TempFixed < 0.0_DP ) ) then
      if ( .not. associated( phy_adj_fix % xyz_TempFixed ) ) then
        allocate( phy_adj_fix % xyz_TempFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      end if
      phy_adj_fix % xyz_TempFixed = phy_adj_fix % TempFixed
    end if

    if ( .not. ( phy_adj_fix % QVapFixed < 0.0_DP ) ) then
      if ( .not. associated( phy_adj_fix % xyz_QVapFixed ) ) then
        allocate( phy_adj_fix % xyz_QVapFixed (0:imax-1,0:jmax-1,0:kmax-1) )
      end if
      phy_adj_fix % xyz_QVapFixed = phy_adj_fix % QVapFixed
    end if

    !-----------------------------------------------------------------
    !  ǡϤν
    !  Initialize data output
    !-----------------------------------------------------------------

    if ( associated( phy_adj_fix % xyz_TempFixed ) ) then
      !-------------------------
      !  xyz_DTempDtFixed ν
      !  Configure the settings for "xyz_DTempDtFixed" output
      name = 'DTempDtFixed'
      longname = 'temperature tendency by adjustment to fixed value'
      units = 'K s-1'
      allocate( dims(4) )
      dims = StoA( 'lon', 'lat', 'sig', 'time' )

      ! ϥեν.
      !   * gthist (gt4_history#GT_HISTORY) ꤵ.
      ! Initialize output file.
      !   * "gthist" (gt4_history#GT_HISTORY) is configured.
      call output_init  ! ֥롼. This is an internal subroutine
      deallocate( dims )
    end if

    if ( associated( phy_adj_fix % xyz_QVapFixed ) ) then
      !-------------------------
      !  xyz_DQVapDtFixed ν
      !  Configure the settings for "xyz_DQVapDtFixed" output
      name = 'DQVapDtFixed'
      longname = 'specific humidity tendency by adjustment to fixed value'
      units = 'kg kg-1 s-1'
      allocate( dims(4) )
      dims = StoA( 'lon', 'lat', 'sig', 'time' )

      ! ϥեν.
      !   * gthist (gt4_history#GT_HISTORY) ꤵ.
      ! Initialize output file.
      !   * "gthist" (gt4_history#GT_HISTORY) is configured.
      call output_init  ! ֥롼. This is an internal subroutine
      deallocate( dims )
    end if

!!$    !-------------------------
!!$    !  y_Data2 ν
!!$    !  Configure the settings for "y_Data2" output
!!$    name = 'Data2'
!!$    longname = 'Sample data (2)'
!!$    units = '1'
!!$    allocate( dims(2) )
!!$    dims = StoA( 'lat', 'time' )
!!$
!!$    ! ϥեν.
!!$    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
!!$    ! Initialize output file.
!!$    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
!!$    call output_init  ! ֥롼. This is an internal subroutine
!!$    ! °ղäʤɤԤˤϰʲΤ褦ˤ. 
!!$    ! Describe codes as follows in order to add attributes etc.
!!$    if ( associated( gthist ) ) then
!!$      call HistoryAddAttr( &
!!$        & history = gthist, &                           ! (inout)
!!$        & varname = name, attrname = 'missing_value', & ! (in)
!!$        &   value = 9999.0 )                            ! (in)
!!$    end if
!!$
!!$    deallocate( dims )

    !-----------------------------------------------------------------
    !  Υ⥸塼뤫Ϥѿ̾ΥꥹȤɽ
    !  Print list of names of variables output from this module
    !-----------------------------------------------------------------
    call Printf( STDOUT, &
      & ' *** MESSAGE *** +---- "%c" module output varnames list -----', &
      & c1 = 'phy_adjust_fixed' )
    call DCHashRewind( hashv = registered_varnames ) ! (inout)
    do
      call DCHashNext( hashv = registered_varnames, &    ! (inout)
        & key = name, value = longname, end = hash_end ) ! (out)
      if ( hash_end ) exit
      call Printf( STDOUT, &
        & ' *** MESSAGE *** |      "%c" (%c)', &
        & c1 = trim(name), c2 = trim(longname) )
    enddo
    call DCHashDelete( hashv = registered_varnames ) ! (inout)
    call Printf( STDOUT, &
      & ' *** MESSAGE *** `----------------------------------------' )

    !-----------------------------------------------------------------
    !  ̵ѿ̾Υå
    !  Check invalid variable names
    !-----------------------------------------------------------------
    call HstNmlInfoAllNameValid( &
      & gthstnml = phy_adj_fix % gthstnml, &               ! (inout)
      & invalid = allvar_invalid, names = names_invalid, & ! (out)
      & err = err )                                        ! (out)

    if ( allvar_invalid ) then
      stat = HST_EBADVARNAME
      cause_c = names_invalid
      phy_adj_fix % initialized = .true.
      call PhyAdjFixClose( phy_adj_fix, & ! (inout)
        & err = err )                     ! (out)
      if ( present(nmlfile) ) then
        nmlfile_msg = ' or NAMELIST "phy_adjust_fixed_history_nml" in "'// trim(nmlfile) // '"'
      else
        nmlfile_msg = ''
      end if
      call MessageNotify( 'W', subname, &
        & 'names "%c" from an optional argument "history_varlist"%c are invalid.', &
        & c1 = trim(names_invalid), &
        & c2 = trim(nmlfile_msg) )
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ͤΥå
    !  Validate setting values
    !-----------------------------------------------------------------
!!$    if ( phy_adj_fix % CoefAlpha < 0.0_DP ) then
!!$      stat = DC_ENEGATIVE
!!$      cause_c = 'CoefAlpha'
!!$      goto 999
!!$    end if

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    phy_adj_fix % initialized = .true.
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  contains

    subroutine output_init
      !
      ! ѿ *name* ˴ؤƽϥեνԤޤ. 
      ! ϥե̾ϴֳ֤ʤɤξ phy_adj_fix % gthstnml 
      ! Фޤ. 
      ! 
      ! ѿ *name* ˴ؤƽϤԤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ϥǡ٤ precision , 
      ! ϥǡʿѲβݤ average ꤷޤ. 
      ! 
      ! ɸϤɽѿꥹ *registered_varnames* 
      ! *name*, *longname*, *dims*, *units* Ͽޤ. 
      !
      ! An output file is initialized for a variable *name*. 
      ! Information such as the output filename and output intervals 
      ! is taken out of "phy_adj_fix % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with the "gt4_history#GT_HISTORY" variable of 
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, the accuracy of output data is set to *precision*, and 
      ! right or wrong of averaging the output data is set to *average*. 
      !
      ! *name*, *longname*, *dims*, *units* are registered to 
      ! a list of variables *registered_varnames* that is printed to 
      ! standard output. 
      !
      use dc_date, only: DCDiffTimeCreate, EvalSec, EvalByUnit
      use gt4_history_nmlinfo, only: GTHST_NMLINFO, &
        & HstNmlInfoInitialized, HstNmlInfoInquire, &
        & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine, &
        & HstNmlInfoSetValidName
      use gt4_history, only: GT_HISTORY, &
        & HistoryCreate, HistoryAddVariable, HistoryPut, &
        & HistoryAddAttr, HistoryInitialized

      !-----------------------------------
      !  ѿ
      !  Work variables
      character(STRING):: file
                                ! ҥȥǡΥե̾. 
                                ! History data filenames
      character(STRING):: dims_str
                                ! ɸΥꥹ. 
                                ! List of axes
      real:: interval_value
                                ! ҥȥǡνϴֳ֤ο. 
                                ! Numerical value for interval of history data output
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
      real(DP), parameter:: PI = 3.1415926535897930_DP
                                ! $ \pi $ . ߼Ψ. Circular constant
    continue
      !-----------------------------------------------------------------
      !  ɸϤɽѿϿ
      !  Register a variable name for print to standard output
      !-----------------------------------------------------------------
      if ( allocated(dims) ) then
        dims_str = JoinChar( dims, ',' )
      else
        dims_str = ''
      end if
      call DCHashPut( hashv = registered_varnames, &     ! (inout)
        & key = name, &                                  ! (in)
        & value = trim( longname ) // ' [' // &
        &         trim( units ) // '] {' // &
        &         trim( dims_str ) // '}' ) ! (in)

      !-----------------------------------------------------------------
      !  ѿν
      !  Initialize variable
      !-----------------------------------------------------------------
      nullify( gthist )
      precision = 'float'
      average = .false.

      !-----------------------------------------------------------------
      !  ѿ̾ͭ
      !  Set validation of the variable name
      !-----------------------------------------------------------------
      call HstNmlInfoSetValidName( &
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name )                        ! (in)

      !-----------------------------------------------------------------
      !  Ϥͭɤǧ
      !  Confirm whether the output is effective
      !-----------------------------------------------------------------
      if ( .not. HstNmlInfoOutputValid( phy_adj_fix % gthstnml, name ) ) then
        return
      end if

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿμ
      !  Get "GT_HISTORY" variable
      !-----------------------------------------------------------------
      call HstNmlInfoAssocGtHist( &
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & history = gthist, &                  ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      call HstNmlInfoInquire( &
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & precision = precision, &             ! (out)
        & average = average, &                 ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿνγǧ
      !  Check initialization of "GT_HISTORY" variable
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then

        !---------------------------------------------------------------
        !  HistoryAddVariable ˤѿ
        !  A variable is created by "HistoryAddVariable"
        !---------------------------------------------------------------
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
        return
      end if

      !-----------------------------------------------------------------
      !  HistoryCreate Τͤμ
      !  Get the settings for "HistoryCreate"
      !-----------------------------------------------------------------
      call HstNmlInfoInquire( &
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & file = file, &                       ! (out)
        & interval_unit = interval_unit, &     ! (out)
        & interval_value = interval_value, &   ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  HistoryCreate ˤե
      !  Files are created by "HistoryCreate"
      !-----------------------------------------------------------------
      call HistoryCreate( &
        & history = gthist, &   ! (out)
        & file = file, &        ! (in)
        & title = 'Adjustment of some physical values to a certain fixed values', &  ! (in)
        & source = 'dcpam4 project : ' // trim(version), &       ! (in)
        & institution = 'GFD Dennou Club', &                     ! (in)
        & dims = StoA( 'lon', 'lat', 'sig', 'time' ), &          ! (in)
        & dimsizes = (/ phy_adj_fix % imax, phy_adj_fix % jmax, &
        &               phy_adj_fix % kmax, 0 /), & ! (in)
        & longnames = StoA('longitude', 'latitude', &
        &                  'sigma at layer midpoints', &
        &                  'time'), &                            ! (in)
        & units = StoA( 'degree_east', 'degree_north', &
        &               '1', interval_unit ), &                  ! (in)
        & origin = real( EvalbyUnit( phy_adj_fix % current_time, &
        &                            interval_unit) ), &         ! (in)
        & interval = interval_value, &                           ! (in)
        & err = err )                                            ! (out)
      if ( present_and_true( err ) ) then
        nullify( gthist )
        return
      end if

      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lon', attrname = 'standard_name', & ! (in)
        & value = 'longitude' )                          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lat', attrname = 'standard_name', & ! (in)
        & value = 'latitude' )                           ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'standard_name', &   ! (in)
        & value = 'atmosphere_sigma_coordinate' )          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                             ! (inout)
        & varname = 'time', attrname = 'standard_name', & ! (in)
        & value = 'time' )                                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'positive', &        ! (in)
        & value = 'down' )                                 ! (in)

      call HistoryPut( &
        & history = gthist, &                           ! (inout)
        & varname = 'lon', &                            ! (in)
        & array = phy_adj_fix % x_Lon / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, &                           ! (inout)
        & varname = 'lat', &                            ! (in)
        & array = phy_adj_fix % y_Lat / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, &             ! (inout)
        & varname = 'sig', &              ! (in)
        & array = phy_adj_fix % z_Sigma ) ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon_weight', &                           ! (in)
        & dims = StoA('lon'), &                               ! (in)
        & longname = 'weight for integration in longitude', & ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lon_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lon_weight', &               ! (in)
        & array = phy_adj_fix % x_Lon_Weight )    ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat_weight', &                           ! (in)
        & dims = StoA('lat'), &                               ! (in)
        & longname = 'weight for integration in latitude', &  ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lat_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lat_weight', &               ! (in)
        & array = phy_adj_fix % y_Lat_Weight )    ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'sig_weight', &                           ! (in)
        & dims = StoA('sig'), &                               ! (in)
        & longname = 'weight for integration in sigma', &     ! (in)
        & units = '1', xtype = 'float' )                      ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'sig', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'sig_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'sig_weight', &               ! (in)
        & array = phy_adj_fix % z_DelSigma )      ! (in)

      !-----------------------------------------------------------------
      !  HistoryAddVariable ˤѿ
      !  A variable is created by "HistoryAddVariable"
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
      else
        nullify( gthist )
      end if

    end subroutine output_init

  end subroutine PhyAdjFixCreate

  subroutine AdjustFixed( phy_adj_fix, &
    & xyz_Temp, xyz_QVap, &
    & xyz_DTempDtFixed, xyz_DQVapDtFixed, &
    & historyput_flag, &
    & err )
    !
    ! Ĥʪ (,  *xyz_Temp*, 漾 *xyz_QVap*) 
    ! 򤢤 (PhyAdjFixCreate ǻ) ˸ꤷޤ. 
    ! ץʥ *xyz_DTempDtFixed*, *xyz_DQVapDtFixed* 
    ! Ϳ, ѿѲ̤֤ޤ. 
    ! ׻, *phy_adj_fix* λ򹹿ޤ. 
    !
    ! ʤ, Ϳ줿 *phy_adj_fix*  PhyAdjFixCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Fix some physical values (now, only temperature *xyz_Temp*,
    ! and specific humidity *xyz_QVap*) to a certain value 
    ! (this is configured by "PhyAdjFixCreate"). 
    ! When optional arguments *xyz_DTempDtFixed*, *xyz_DQVapDtFixed* are
    ! given, tendencies are returned to them.
    ! After the calculation, time stored in *phy_adj_fix* is updated. 
    !
    ! If *phy_adj_fix* is not initialized 
    ! by "PhyAdjFixCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_present, only: present_and_true
    use dc_date, only: mod, operator(+), operator(==), EvalbyUnit, EvalSec
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT, USR_ERRNO
    use dc_message, only: MessageNotify
    use gt4_history_nmlinfo, only: HstNmlInfoInquire, &
      & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistoryPut, HistoryInitialized
    implicit none
    type(PHYADJFIX), intent(inout):: phy_adj_fix
    real(DP), intent(inout), optional:: xyz_Temp (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ T $ .     
                              ! Temperature
    real(DP), intent(inout), optional:: xyz_QVap (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ q $ .     漾
                              ! Specific humidity
    real(DP), intent(out), optional:: xyz_DTempDtFixed (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ DP{T}{t} $ .
                              ! ͤؤĴ˴ؤ벹٤ѲΨ
                              ! Temperature tendency by adjustment to fixed value
    real(DP), intent(out), optional:: xyz_DQVapDtFixed (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ q $ .
                              ! ͤؤĴ˴ؤ漾ѲΨ
                              ! Specific humidity tendency by adjustment to fixed value
    logical, intent(in), optional:: historyput_flag
                              ! ǡϤΥե饰. 
                              ! SetTime ˤäƻŪ
                              ! ꤷˤ, ΰ
                              ! .true. ޤ .false. ꤹ
                              ! ȤǥǡϤΥ󥪥դ
                              ! Ū˻ꤹɬפޤ. 
                              ! ǥեȤ .false. Ǥ. 
                              ! 
                              ! Data output flag. 
                              ! When time is specified by "SetTime", 
                              ! explicit specification of data output 
                              ! on/off by specifying ".true." or ".false." 
                              ! to this argument. 
                              ! Default value is ".false.". 
                              ! 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  *phy_adj_fix* Ф
    !  Setting values fetched from *phy_adj_fix*
    real(DP):: DelTime        ! $ \Delta t $ . ॹƥå. Time step
    real(DP):: xyz_TempFixed (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ T_0 $ .     ٤θ. 
                              ! Fixed value of temperature
    real(DP):: xyz_QVapFixed (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ q_0 $ .     漾θ. 
                              ! Fixed value of specific humidity

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    real:: time
                              ! . Time
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP):: xyz_DTempDtFixedWork (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ DP{T}{t} $ .
                              ! ͤؤĴ˴ؤ벹٤ѲΨ
                              ! Temperature tendency by adjustment to fixed value
    real(DP):: xyz_DQVapDtFixedWork (0:phy_adj_fix%imax-1,0:phy_adj_fix%jmax-1,0:phy_adj_fix%kmax-1)
                              ! $ q $ .
                              ! ͤؤĴ˴ؤ漾ѲΨ
                              ! Specific humidity tendency by adjustment to fixed value

    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'AdjustFixed'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_adj_fix % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYADJFIX'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  *phy_adj_fix* ˳ǼƤͤμФ
    !  Fetch setting values stored in *phy_adj_fix*
    !-----------------------------------------------------------------
    DelTime       = EvalSec( phy_adj_fix % delta_time )
    if ( associated( phy_adj_fix % xyz_TempFixed ) ) xyz_TempFixed = phy_adj_fix % xyz_TempFixed
    if ( associated( phy_adj_fix % xyz_QVapFixed ) ) xyz_QVapFixed = phy_adj_fix % xyz_QVapFixed

    !-----------------------------------------------------------------
    !  ٤Ĵ
    !  Adjustment of temperature
    !-----------------------------------------------------------------
    xyz_DTempDtFixedWork = 0.0_DP
    if ( present( xyz_Temp ) ) then
      if ( associated( phy_adj_fix % xyz_TempFixed ) ) then
        xyz_DTempDtFixedWork = ( xyz_TempFixed - xyz_Temp ) / DelTime
        xyz_Temp = xyz_TempFixed
      else
        call MessageNotify( 'W', subname, &
          & 'fixed value of "%c" is not specified.', &
          & c1 = 'Temp' )
      end if
    end if
    if ( present( xyz_DTempDtFixed ) ) then
      xyz_DTempDtFixed = xyz_DTempDtFixedWork
    end if

    !-----------------------------------------------------------------
    !  漾Ĵ
    !  Adjustment of specific humidity
    !-----------------------------------------------------------------
    xyz_DQVapDtFixedWork = 0.0_DP
    if ( present( xyz_QVap ) ) then
      if ( associated( phy_adj_fix % xyz_QVapFixed ) ) then
        xyz_DQVapDtFixedWork = ( xyz_QVapFixed - xyz_QVap ) / DelTime
        xyz_QVap = xyz_QVapFixed
      else
        call MessageNotify( 'W', subname, &
          & 'fixed value of "%c" is not specified.', &
          & c1 = 'QVap' )
      end if
    end if
    if ( present( xyz_DQVapDtFixed ) ) then
      xyz_DQVapDtFixed = xyz_DQVapDtFixedWork
    end if

    !----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  History data output
    !----------------------------------------------------------------

    if ( associated( phy_adj_fix % xyz_TempFixed ) ) then
      !-----------------------------------
      !  xyz_DTempDtFixed ν
      !  Output "xyz_DTempDtFixed"
      name = 'DTempDtFixed'

      ! ϤΥå. 
      !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
      ! Check for output.
      !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
      call output_check ! ֥롼. This is an internal subroutine

      ! ϥǡ array Ϥ.
      ! Give output data to argument "array"
      if ( associated( gthist ) ) then
        call HistoryPut( &
          & history = gthist, &                ! (inout)
          & varname = name, array = xyz_DTempDtFixedWork, & ! (in)
          & time = time, quiet = .false., &    ! (in)
          & err = err )                        ! (out)
      end if
    end if

    if ( associated( phy_adj_fix % xyz_QVapFixed ) ) then
      !-----------------------------------
      !  xyz_DQVapDtFixed ν
      !  Output "xyz_DQVapDtFixed"
      name = 'DQVapDtFixed'

      ! ϤΥå. 
      !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
      ! Check for output.
      !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
      call output_check ! ֥롼. This is an internal subroutine

      ! ϥǡ array Ϥ.
      ! Give output data to argument "array"
      if ( associated( gthist ) ) then
        call HistoryPut( &
          & history = gthist, &                ! (inout)
          & varname = name, array = xyz_DQVapDtFixedWork, & ! (in)
          & time = time, quiet = .false., &    ! (in)
          & err = err )                        ! (out)
      end if
    end if


!!$    !-----------------------------------
!!$    !  y_Data2 ν
!!$    !  Output "y_Data2"
!!$    name = 'Data2'
!!$
!!$    ! ϤΥå. 
!!$    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
!!$    ! Check for output.
!!$    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
!!$    call output_check ! ֥롼. This is an internal subroutine
!!$
!!$    ! ϥǡ array Ϥ.
!!$    ! Give output data to argument "array"
!!$    if ( associated( gthist ) ) then
!!$      call HistoryPut( &
!!$        & history = gthist, &                ! (inout)
!!$        & varname = name, array = y_Data2, & ! (in)
!!$        & time = time, quiet = .false., &    ! (in)
!!$        & err = err )                        ! (out)
!!$    end if

    !-----------------------------------------------------------------
    !  ι
    !  Update time
    !-----------------------------------------------------------------
    phy_adj_fix % current_time = &
      & phy_adj_fix % current_time + phy_adj_fix % delta_time

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  contains

    subroutine output_check
      !
      ! ѿ *name* Ϥ뤫ɤåޤ. 
      ! Ϥ˴ؤ dcm_sam_cod % gthstnml Фޤ. 
      !
      ! ѿ *name* ˴ؤƽϤ褦ꤵƤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ߻ *time* ꤷޤ. 
      !
      ! Check whether to output variable *name*. 
      ! Information about output is taken out of "dcm_sam_cod % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with "gt4_history#GT_HISTORY" variable of
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, current time is set to *time*. 
      !
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
    continue

      nullify( gthist )
      time = 0.0

      if ( HstNmlInfoOutputValid( phy_adj_fix % gthstnml, name ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = phy_adj_fix % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & interval_unit = interval_unit )          ! (out)

        time = real( EvalbyUnit( phy_adj_fix % current_time, interval_unit ) )
        if ( present_and_true( historyput_flag ) ) time = 0.0

        call HstNmlInfoAssocGtHist( & 
          & gthstnml = phy_adj_fix % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & history = gthist, err = err )            ! (out)
        if ( present_and_true( err ) ) then
          nullify( gthist )
          return
        end if

        if ( .not. HistoryInitialized( gthist ) ) nullify( gthist )
      end if

    end subroutine output_check
  end subroutine AdjustFixed

!!$  subroutine PhyAdjFixSample( phy_adj_fix, err )
!!$    !--
!!$    ! PhyAdjFixSample 򵭽ҤƤ. 
!!$    !++
!!$    ! ʤ, Ϳ줿 *phy_adj_fix*  PhyAdjFixCreate 
!!$    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
!!$    !--
!!$    ! Describe brief of "PhyAdjFixSample". 
!!$    !++
!!$    ! If *phy_adj_fix* is not initialized 
!!$    ! by "PhyAdjFixCreate" yet, error is occurred.
!!$    !
!!$    use dc_trace, only: BeginSub, EndSub
!!$    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
!!$    use dc_types, only: DP, STRING, TOKEN, STDOUT
!!$    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
!!$    implicit none
!!$    type(PHYADJFIX), intent(inout):: phy_adj_fix
!!$    logical, intent(out), optional:: err
!!$                              ! 㳰ѥե饰. 
!!$                              ! ǥեȤǤ, μ³ǥ顼
!!$                              ! , ץ϶λޤ. 
!!$                              !  *err* Ϳ, 
!!$                              ! ץ϶λ, 
!!$                              ! *err*  .true. ޤ. 
!!$                              !
!!$                              ! Exception handling flag. 
!!$                              ! By default, when error occur in 
!!$                              ! this procedure, the program aborts. 
!!$                              ! If this *err* argument is given, 
!!$                              ! .true. is substituted to *err* and 
!!$                              ! the program does not abort. 
!!$
!!$    !-----------------------------------
!!$    !  *phy_adj_fix* Ф
!!$    !  Setting values fetched from *phy_adj_fix*
!!$!!$    real(DP):: CoefAlpha      ! $ \alpha $ . . Coefficient
!!$!!$    real(DP):: DelTime        ! $ \Delta t $ . ॹƥå. Time step
!!$
!!$    !-----------------------------------
!!$    !  ѿ
!!$    !  Work variables
!!$    integer:: stat
!!$    character(STRING):: cause_c
!!$    character(*), parameter:: subname = 'PhyAdjFixSample'
!!$  continue
!!$    call BeginSub( subname )
!!$    stat = DC_NOERR
!!$    cause_c = ''
!!$
!!$    !-----------------------------------------------------------------
!!$    !  Υå
!!$    !  Check initialization
!!$    !-----------------------------------------------------------------
!!$    if ( .not. phy_adj_fix % initialized ) then
!!$      stat = DC_ENOTINIT
!!$      cause_c = 'PHYADJFIX'
!!$      goto 999
!!$    end if
!!$
!!$    !-----------------------------------------------------------------
!!$    !  *phy_adj_fix* ˳ǼƤͤμФ
!!$    !  Fetch setting values stored in *phy_adj_fix*
!!$    !-----------------------------------------------------------------
!!$!!$    CoefAlpha = phy_adj_fix % CoefAlpha
!!$!!$    DelTime   = phy_adj_fix % DelTime
!!$
!!$    !-----------------------------------------------------------------
!!$    !  λ, 㳰
!!$    !  Termination and Exception handling
!!$    !-----------------------------------------------------------------
!!$999 continue
!!$    call StoreError( stat, subname, err, cause_c )
!!$    call EndSub( subname )
!!$
!!$  end subroutine PhyAdjFixSample

  subroutine PhyAdjFixClose( phy_adj_fix, err )
    !
    ! PHYADJFIX ѿνλԤޤ. 
    ! ʤ, Ϳ줿 *phy_adj_fix*  PhyAdjFixCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Deconstructor of "PHYADJFIX". 
    ! Note that if *phy_adj_fix* is not initialized 
    ! by "PhyAdjFixCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: HstNmlInfoClose, HstNmlInfoNames, &
      & HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistoryClose, HistoryInitialized
    implicit none
    type(PHYADJFIX), intent(inout):: phy_adj_fix
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhyAdjFixClose'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_adj_fix % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYADJFIX'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  "PHYADJFIX" ξõ
    !  Clear the settings for "PHYADJFIX"
    !-----------------------------------------------------------------
    deallocate( phy_adj_fix % x_Lon )
    deallocate( phy_adj_fix % x_Lon_Weight )
    deallocate( phy_adj_fix % y_Lat )
    deallocate( phy_adj_fix % y_Lat_Weight )
    deallocate( phy_adj_fix % z_Sigma )
    deallocate( phy_adj_fix % z_DelSigma )

    phy_adj_fix % TempFixed = -1.0_DP
    phy_adj_fix % QVapFixed = -1.0_DP
    if ( associated( phy_adj_fix % xyz_TempFixed ) ) deallocate( phy_adj_fix % xyz_TempFixed )
    if ( associated( phy_adj_fix % xyz_QVapFixed ) ) deallocate( phy_adj_fix % xyz_QVapFixed )

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡϤνλ
    !  Terminate the settings for history data output
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( phy_adj_fix % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit
      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)
      if ( HistoryInitialized( gthist ) ) then

        call HistoryClose( history = gthist, & ! (inout)
          & err = err )                        ! (out)

      end if
    end do

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡγղ
    !  Deallocate the settings for history data output
    !-----------------------------------------------------------------
    call HstNmlInfoClose( &
      & phy_adj_fix % gthstnml, & ! (inout)
      & err = err )                   ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    phy_adj_fix % initialized = .false.
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhyAdjFixClose

  subroutine PhyAdjFixPutLine( phy_adj_fix, unit, indent, err )
    !
    !  *phy_adj_fix* ꤵƤޤ. 
    ! ǥեȤǤϥåɸϤ˽Ϥޤ. 
    ! *unit* ֹꤹ뤳Ȥ, ѹ뤳ȤǽǤ. 
    !
    ! Print information of *phy_adj_fix*. 
    ! By default messages are output to standard output. 
    ! Unit number for output can be changed by *unit* argument. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_date, only: EvalSec
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoPutLine
    implicit none
    type(PHYADJFIX), intent(in):: phy_adj_fix
    integer, intent(in), optional:: unit
                              ! ֹ. 
                              ! ǥեȤνɸ. 
                              !
                              ! Unit number for output. 
                              ! Default value is standard output. 
    character(*), intent(in), optional:: indent
                              ! ɽåλ. 
                              !
                              ! Indent of displayed messages. 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: out_unit
    integer:: indent_len
    character(STRING):: indent_str
    character(*), parameter:: subname = 'PhyAdjFixPutLine'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  ֹȻ
    !  Configure output unit number and indents
    !-----------------------------------------------------------------
    if ( present(unit) ) then
      out_unit = unit
    else
      out_unit = STDOUT
    end if

    indent_len = 0
    indent_str = ''
    if ( present(indent) ) then
      if ( len(indent) /= 0 ) then
        indent_len = len(indent)
        indent_str(1:indent_len) = indent
      end if
    end if

    !-----------------------------------------------------------------
    !  "PHYADJFIX" ΰ
    !  Print the settings for "PHYADJFIX"
    !-----------------------------------------------------------------
    if ( phy_adj_fix % initialized ) then
      call Printf( unit = out_unit, &                 ! (in)
        & fmt = indent_str(1:indent_len) // & 
        &       '#<PHYADJFIX:: @initialized=%y', & ! (in)
        & l = (/ phy_adj_fix % initialized /) )   ! (in)

      call Printf( unit = out_unit, &             ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @imax=%d @jmax=%d @kmax=%d', &  ! (in)
        & i = (/ phy_adj_fix % imax, phy_adj_fix % jmax, &
        &        phy_adj_fix % kmax /) )          ! (in)

      call Printf( unit = out_unit, &                             ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @current_time=%f [sec] @delta_time=%f [sec]', & ! (in)
        & d = (/ EvalSec( phy_adj_fix % current_time ), &
        &        EvalSec( phy_adj_fix % delta_time ) /) )      ! (in)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @TempFixed=%f', &         ! (in)
        & d = (/phy_adj_fix % TempFixed/) ) ! (in)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @QVapFixed=%f', &         ! (in)
        & d = (/phy_adj_fix % QVapFixed/) ) ! (in)

      if ( associated( phy_adj_fix % xyz_TempFixed ) ) then
        call PutLine( array = phy_adj_fix % xyz_TempFixed, & ! (in)
          & unit = out_unit, &                           ! (in)
          & lbounds = lbound(phy_adj_fix % xyz_TempFixed), & ! (in)
          & ubounds = ubound(phy_adj_fix % xyz_TempFixed), & ! (in)
          & indent = indent_str(1:indent_len) // &
          &          ' @xyz_TempFixed=' )                        ! (in)
      end if

      if ( associated( phy_adj_fix % xyz_QVapFixed ) ) then
        call PutLine( array = phy_adj_fix % xyz_QVapFixed, & ! (in)
          & unit = out_unit, &                           ! (in)
          & lbounds = lbound(phy_adj_fix % xyz_QVapFixed), & ! (in)
          & ubounds = ubound(phy_adj_fix % xyz_QVapFixed), & ! (in)
          & indent = indent_str(1:indent_len) // &
          &          ' @xyz_QVapFixed=' )                        ! (in)
      end if

      call PutLine( array = phy_adj_fix % x_Lon, & ! (in)
        & unit = out_unit, &                           ! (in)
        & lbounds = lbound(phy_adj_fix % x_Lon), & ! (in)
        & ubounds = ubound(phy_adj_fix % x_Lon), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @x_Lon=' )                        ! (in)

      call PutLine( array = phy_adj_fix % y_Lat, & ! (in)
        & unit = out_unit, &                           ! (in)
        & lbounds = lbound(phy_adj_fix % y_Lat), & ! (in)
        & ubounds = ubound(phy_adj_fix % y_Lat), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @y_Lat=' )                        ! (in)

      call PutLine( array = phy_adj_fix % z_Sigma, & ! (in)
        & unit = out_unit, &                         ! (in)
        & lbounds = lbound(phy_adj_fix % z_Sigma), & ! (in)
        & ubounds = ubound(phy_adj_fix % z_Sigma), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @z_Sigma=' )                    ! (in)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @gthstnml=' )               ! (in)
      call HstNmlInfoPutLine( &
        & gthstnml = phy_adj_fix % gthstnml, &    ! (in)
        & unit = out_unit, &                          ! (in)
        & indent = indent_str(1:indent_len) // '  ' ) ! (in)

      call Printf( unit = out_unit, &             ! (in)
        & fmt = indent_str(1:indent_len) // '>' ) ! (in)
    else
      call Printf( unit = out_unit, &                   ! (in)
        & fmt = indent_str(1:indent_len) // &           ! (in)
        &       '#<PHYADJFIX:: @initialized=%y>', &  ! (in)
        & l = (/phy_adj_fix % initialized/) )       ! (in)
    end if

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhyAdjFixPutLine

  logical function PhyAdjFixInitialized( phy_adj_fix ) result(result)
    !
    ! *phy_adj_fix* ꤵƤˤ .true. , 
    ! ꤵƤʤˤ .false. ֤ޤ. 
    !
    ! If *phy_adj_fix* is initialized, .true. is returned. 
    ! If *phy_adj_fix* is not initialized, .false. is returned. 
    !
    implicit none
    type(PHYADJFIX), intent(in):: phy_adj_fix
  continue
    result = phy_adj_fix % initialized
  end function PhyAdjFixInitialized

  subroutine PhyAdjFixSetTime( phy_adj_fix, &
    & current_time_value, current_time_unit, &
    & err )
    !
    ! *phy_adj_fix* ФƻԤޤ. 
    !
    ! ҥȥǡϤƤˤ, 
    ! ҥȥǡνϻꤷޤ. 
    ! ٤Ǥ⤳Υ֥롼Ƥˤ, 
    ! ʸΥҥȥǡˤΥ֥롼ƤӽФ, 
    ! ԤäƤ. 
    ! ޤ, ǡϤ륵֥롼ФƤ 
    ! ץʥ historyput_flag  .true. ͿƤ. 
    !
    ! ʤ, Ϳ줿 *phy_adj_fix*  PhyAdjFixCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Set time to *phy_adj_fix*. 
    !
    ! When history data are output, 
    ! the output time of history data are specified. 
    ! Once this subroutine is called, the time of history data must be 
    ! specified by this routine before history data output. 
    ! In additional, give ".true." to an optional argument 
    ! "historyput_flag" of a data output subroutine. 
    !
    ! If *phy_adj_fix* is not initialized 
    ! by "PhyAdjFixCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_date, only: DCDiffTimeCreate, EvalbyUnit
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: HstNmlInfoAdd, HstNmlInfoInquire, &
      & HstNmlInfoNames, HstNmlInfoAssocGtHist, &
      & HstNmlInfoOutputStepDisable, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistorySetTime, HistoryInitialized
    implicit none
    type(PHYADJFIX), intent(inout):: phy_adj_fix
    real, intent(in):: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in):: current_time_unit
                              ! ߻ñ. Unit of current time
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhyAdjFixSetTime'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_adj_fix % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYADJFIX'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  
    !  Configure time
    !-----------------------------------------------------------------
    call DCDiffTimeCreate( &
      & diff = phy_adj_fix % current_time, &  ! (out)
      & value = real( current_time_value, DP ), & ! (in)
      & unit = current_time_unit )                ! (in)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡλ
    !  Configure the time of history data
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( phy_adj_fix % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit

      call HstNmlInfoOutputStepDisable( & 
        & gthstnml = phy_adj_fix % gthstnml, & ! (inout)
        & name = name, &                           ! (in)
        & err = err )                              ! (out)

      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = phy_adj_fix % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)

      if ( HistoryInitialized( gthist ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = phy_adj_fix % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & interval_unit = interval_unit )          ! (out)

        call HistorySetTime( &
          & history = gthist, &                     ! (inout)
          & time = &
          &   real( EvalbyUnit( phy_adj_fix % current_time, &
          &                     interval_unit) ) ) ! (in)
      end if

    end do

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhyAdjFixSetTime

  subroutine PhyAdjFixNmlRead( nmlfile, &
    & TempFixed, QVapFixed, &
    & gthstnml, &
    & err )
    !
    ! NAMELIST ե *nmlfile* ͤϤ뤿
    ! ֥롼Ǥ. PhyAdjFixCreate 
    ! ǸƤӽФ뤳ȤꤷƤޤ. 
    !
    ! ͤ NAMELIST եǻꤵƤʤˤ, 
    ! Ϥ줿ͤΤޤ֤ޤ. 
    !
    ! ʤ, *nmlfile* ˶ʸͿ줿, ޤ
    ! Ϳ줿 *nmlfile* ɤ߹ळȤǤʤ, 
    ! ץϥ顼ȯޤ. 
    !
    ! This is a subroutine to input values from 
    ! NAMELIST file *nmlfile*. This subroutine is expected to be 
    ! called by "PhyAdjFixCreate". 
    !
    ! A value not specified in NAMELIST file is returned 
    ! without change. 
    !
    ! If *nmlfile* is empty, or *nmlfile* can not be read, 
    ! error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_iounit, only: FileOpen
    use dc_message, only: MessageNotify
    use dc_present, only: present_and_true
    use dc_date, only: DCDiffTimeCreate
    use dc_error, only: StoreError, DC_NOERR, DC_ENOFILEREAD, DC_ENOTINIT
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoAdd, &
      & HstNmlInfoInquire, HstNmlInfoInitialized, HstNmlInfoPutLine
    implicit none
    character(*), intent(in):: nmlfile
                              ! NAMELIST ե̾. 
                              ! NAMELIST file name
    real(DP), intent(inout):: TempFixed
                              ! $ T_0 $ .     ٤θ (Ѳ̵). 
                              ! Fixed value of temperature (spatial variation is nothing)
    real(DP), intent(inout):: QVapFixed
                              ! $ q_0 $ .     漾θ (Ѳ̵). 
                              ! Fixed value of specific humidity (spatial variation is nothing)
    type(GTHST_NMLINFO), intent(inout):: gthstnml
                              ! NAMELIST#phy_adjust_fixed_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! ǥեͤʤɤ
                              ! ԤäͿ뤳. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#phy_adjust_fixed_history_nml". 
                              ! 
                              ! Before this argument is given to 
                              ! this procedure, initialize and 
                              ! configure the defaut settings. 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    namelist /phy_adjust_fixed_nml/ &
      & TempFixed, QVapFixed
                              ! phy_adjust_fixed ⥸塼
                              ! NAMELIST ѿ̾. 
                              !
                              ! phy_adjust_fixed#PhyAdjFixCreate 
                              ! Ѥݤ, ץʥ *nmlfile* 
                              !  NAMELIST ե̾ꤹ뤳Ȥ, 
                              ! Υե뤫餳 NAMELIST ѿ
                              ! ɤ߹ߤޤ. 
                              !
                              ! NAMELIST group name for 
                              ! "phy_adjust_fixed" module. 
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* when 
                              ! "phy_adjust_fixed#PhyAdjFixCreate" 
                              ! is used, this NAMELIST group is 
                              ! loaded from the file. 

    character(STRING):: name
                              ! ѿ̾. 
                              ! ξˤ, ¾ͤ
                              ! phy_adjust_fixed ⥸塼ˤ
                              ! ϤǡƤ
                              ! ǥեͤȤʤޤ. 
                              ! 
                              ! "Data1,Data2" Τ褦˥ޤǶڤäʣ
                              ! ѿꤹ뤳ȤǽǤ. 
                              ! 
                              ! Variable identifier. 
                              ! If blank is given, other values are 
                              ! used as default values of output data 
                              ! in "phy_adjust_fixed". 
                              ! 
                              ! Multiple variables can be specified 
                              ! as "Data1,Data2" too. Delimiter is comma. 
    character(STRING):: file
                              ! ϥե̾. 
                              ! ϥǥեͤȤƤϻѤޤ. 
                              ! *name* ͤꤵƤΤͭǤ. 
                              ! 
                              ! Output file name. 
                              ! This is not used as default value. 
                              ! This value is valid only when *name* is 
                              ! specified. 

    real:: interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! ͤͿ, Ϥ޻ߤޤ. 
                              ! Numerical value for interval of history data output
                              ! Negative values suppresses output.
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    character(STRING):: fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefixes of history data filenames

    namelist /phy_adjust_fixed_history_nml/ &
      & name, &
      & file, &
      & interval_value, &
      & interval_unit, &
      & precision, &
      & fileprefix, &
      & average
                              ! phy_adjust_fixed ⥸塼Υҥȥǡ
                              ! NAMELIST ѿ̾. 
                              !
                              ! phy_adjust_fixed#PhyAdjFixCreate 
                              ! Ѥݤ, ץʥ *nmlfile* 
                              !  NAMELIST ե̾ꤹ뤳Ȥ, 
                              ! Υե뤫餳 NAMELIST ѿ
                              ! ɤ߹ߤޤ. 
                              !
                              ! NAMELIST group name for 
                              ! history data of "phy_adjust_fixed" module. 
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* when 
                              ! "phy_adjust_fixed#PhyAdjFixCreate" 
                              ! is used, this NAMELIST group is 
                              ! loaded from the file. 

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: unit_nml        ! NAMELIST ե륪ץֹ. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT. 
                              ! IOSTAT of NAMELIST read
    character(TOKEN):: pos_nml
                              ! NAMELIST ɤ߹߻Υե. 
                              ! File position of NAMELIST read
    character(*), parameter:: subname = 'PhyAdjFixNmlRead'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------

    !-----------------------------------------------------------------
    !  ʸ NAMELIST ѿ
    !  Substitute character arguments to NAMELIST group
    !-----------------------------------------------------------------
!!$    key00       = key00_

    !----------------------------------------------------------------
    !  NAMELIST եΥץ
    !  Open NAMELIST file
    !----------------------------------------------------------------
    call FileOpen( unit = unit_nml, & ! (out)
      & file = nmlfile, mode = 'r', & ! (in)
      & err = err )                   ! (out)
    if ( present_and_true(err) ) then
      stat = DC_ENOFILEREAD
      cause_c = nmlfile
      goto 999
    end if

    !-----------------------------------------------------------------
    !  NAMELIST ѿμ
    !  Get NAMELIST group
    !-----------------------------------------------------------------

    !-------------------------
    !  ʤɤμ
    !  Get coefficients etc.
    rewind( unit_nml )
    read( unit = unit_nml, &         ! (in)
      &    nml = phy_adjust_fixed_nml, & ! (out)
      & iostat = iostat_nml )        ! (out)
    if ( iostat_nml == 0 ) then
      call MessageNotify( 'M', subname, &
        & 'NAMELIST group "%c" is loaded from "%c".', &
        & c1 = 'phy_adjust_fixed_nml', c2 = trim(nmlfile) )
      write(STDOUT, nml = phy_adjust_fixed_nml)
    else
      call MessageNotify( 'W', subname, &
        & 'NAMELIST group "%c" is not found in "%c" (iostat=%d).', &
        & c1 = 'phy_adjust_fixed_nml', c2 = trim(nmlfile), &
        & i = (/iostat_nml/) )
    end if

    !-------------------------
    !  ϥǡθ̾μ
    !  Get individual information of output data
    rewind( unit_nml )
    iostat_nml = 0
    pos_nml = ''
    do while ( trim(pos_nml) /= 'APPEND' .and. iostat_nml == 0 )

      name                   = ''
      file                   = ''
      call HstNmlInfoInquire( &
        & gthstnml = gthstnml, &             ! (in)
        & interval_value = interval_value, & ! (out)
        & interval_unit = interval_unit, &   ! (out)
        & precision = precision, &           ! (out)
        & average = average, &               ! (out)
        & fileprefix = fileprefix )          ! (out)

      read( unit = unit_nml, &                 ! (in)
        &    nml = phy_adjust_fixed_history_nml, & ! (out)
        & iostat = iostat_nml )                ! (out)
      inquire( unit = unit_nml, & ! (in)
        &  position = pos_nml )   ! (out)

      if ( iostat_nml == 0 ) then
        call MessageNotify( 'M', subname, &
          & 'NAMELIST group "%c" is loaded from "%c".', &
          & c1='phy_adjust_fixed_history_nml', c2=trim(nmlfile) )
        write(STDOUT, nml = phy_adjust_fixed_history_nml)

        call HstNmlInfoAdd( &
          & gthstnml = gthstnml, &             ! (in)
          & name = name, &                     ! (in)
          & file = file, &                     ! (in)
          & interval_value = interval_value, & ! (in)
          & interval_unit = interval_unit, &   ! (in)
          & precision = precision, &           ! (in)
          & average = average, &               ! (in)
          & fileprefix = fileprefix )          ! (in)

      else
        call MessageNotify( 'W', subname, &
          & 'NAMELIST group "%c" is not found in "%c" any more (iostat=%d).', &
          & c1='phy_adjust_fixed_history_nml', c2=trim(nmlfile), &
          & i = (/iostat_nml/) )
      end if
    end do

    close( unit_nml )

    !-----------------------------------------------------------------
    !  NAMELIST ѿʸ
    !  Substitute NAMELIST group to character arguments
    !-----------------------------------------------------------------
!!$    key00_       = key00

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhyAdjFixNmlRead

end module phy_adjust_fixed
